# -*- perl -*-
#
# Test::MemMonitor::Dumper by Daniel Berrange <berrange@redhat.com>
#
# Copyright (C) 2006 Red Hat <http://redhat.com/>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

=pod

=head1 NAME

Test::MemMonitor::Dumper - XYZ

=head1 SYNOPSIS

  use Test::MemMonitor:Dumper;



=head1 DESCRIPTION

This module provides....

=head1 METHODS

=over 4

=cut

package Test::MemMonitor::Dumper;

use strict;
use warnings;

require Exporter;

our @ISA = qw(Exporter);

our %EXPORT_TAGS = (all => [qw /dump_summary dump_groupings dump_mappings/]);
Exporter::export_ok_tags('all');

sub dump_summary {
    my $snapshot = shift;
    
    print "Summary:\n";
    print "\n";
    print "  Mapped:   ", &format_size($snapshot->total_size), "\n";
    print "  Free:     ", &format_size($snapshot->free_size), "\n";
    print "  Virtual:  ", &format_size($snapshot->virtual_size), "\n";
    print "\n";
    print "  Resident: ", &format_size($snapshot->resident_size), "\n";
    print "    Shared:   ", &format_size($snapshot->shared_size), "\n";
    print "      Clean: ", &format_size($snapshot->shared_clean), "\n";
    print "      Dirty: ", &format_size($snapshot->shared_dirty), "\n";
    print "    Private:   ", &format_size($snapshot->private_size), "\n";
    print "      Clean: ", &format_size($snapshot->private_clean), "\n";
    print "      Dirty: ", &format_size($snapshot->private_dirty), "\n";
    print "  Unmapped: ", &format_size($snapshot->unmapped_size), "\n";
    print "\n";
    print "  Text:     ", &format_size($snapshot->text_size), "\n";
    print "  Library:  ", &format_size($snapshot->library_size), "\n";
    print "  Data:     ", &format_size($snapshot->data_size), "\n";
    print "\n";
}

sub dump_groupings {
    my $snapshot = shift;
    my %options = @_;

    my %sorts = (
	"name" => sub { $a->name cmp $b->name },
	"size" => sub { $a->size <=> $b->size },
	"rsize" => sub { $b->size <=> $a->size },
    );

    my $sortType = exists $options{sort} ? $options{sort} : "name";
    $sortType = "name" unless exists $sorts{$sortType};
    my $sort = $sorts{$sortType};

    print "Groupings:\n";
    print "\n";
    printf "  %-60s: %10s      %15s     %15s\n", "", "", "Shared", "Private";
    printf "  %-60s:  %7s        %7s     %7s    %7s    %7s\n", "", "Mapped", "Clean", "Dirty","Clean", "Dirty";
    printf "                                                               ------------------------------------------------------------\n";
    foreach my $grouping (sort { &$sort } $snapshot->groupings) {
	next if $grouping->name eq "[free]" 
	    && not $options{showfree};

	printf "  %-60s: %10s      %10s %10s %10s %10s\n", $grouping->name, &format_size($grouping->size),
	  &format_size($grouping->shared_clean), &format_size($grouping->shared_dirty),
	  &format_size($grouping->private_clean), &format_size($grouping->private_dirty);
    }
    print "\n";

}

sub dump_mappings {
    my $snapshot = shift;
    my %options = @_;
    my %sorts = (
	"addr" => sub { $a->start_address <=> $b->start_address },
	"raddr" => sub { $b->start_address <=> $a->start_address },
	"name" => sub { $a->path cmp $b->path },
	"size" => sub { $a->size <=> $b->size },
	"rsize" => sub { $b->size <=> $a->size },
    );

    my $sortType = exists $options{sort} ? $options{sort} : "addr";
    $sortType = "addr" unless exists $sorts{$sortType};
    my $sort = $sorts{$sortType};
    
    print "Mappings\n";
    print "\n";
    printf "  %-60s  %10s      %15s     %15s\n", "", "", "Shared", "Private";
    printf "  %-60s   %7s        %7s     %7s    %7s    %7s\n", "", "Mapped", "Clean", "Dirty","Clean", "Dirty";
    printf "                                                               ------------------------------------------------------------\n";
    foreach my $mapping (sort { &$sort } $snapshot->mappings) {
	next if $mapping->path eq "[free]" 
	    && not $options{showfree};

	printf "  %-60s: %10s      %10s %10s %10s %10s\n", $mapping->path, &format_size($mapping->size),
	  &format_size($mapping->shared_clean), &format_size($mapping->shared_dirty),
	  &format_size($mapping->private_clean), &format_size($mapping->private_dirty);
    }
    print "\n";
}

sub format_size {
    my $size = shift;
    
    if ($size < 2048) {
	return sprintf "%d bytes", $size;
    } elsif ($size < (1024*1024*2)) {
	return sprintf "%.2f kb", ($size/1024);
    } elsif ($size < (1024*1024*1024*2)) {
	return sprintf "%.2f MB", ($size/1024/1024);
    } else {
	return sprintf "%.2f GB", ($size/1024/1024/1024);
    }    
}


1;

=pod

=back

=head1 AUTHORS

Daniel P. Berrange <berrange@redhat.com>

=head1 LICENSE / COPYRIGHT

Copyright (C) 2006 Red Hat

This software can be distributed under the terms of the GPL v2 or later

=head1 SEE ALSO

=cut

